import xmlrpc.client

from django.conf import settings

ADDRESS_FIELDS = [
    "id",
    "name",
    "street",
    "street2",
    "city",
    "zip",
    "state_id",
    "country_id",
    "email",
    "phone",
    "vat",
    "company_type",
    "type",
]


class OdooClient:
    def __init__(self):
        self.url = settings.ODOO["URL"]
        self.db = settings.ODOO["DB"]
        self.username = settings.ODOO["USERNAME"]
        self.password = settings.ODOO["PASSWORD"]

        self.common_proxy = None
        self.models_proxy = None
        self.uid = None

    def _connect(self):
        """This method is called on the first client request, not on instantiation,
        so that we can instantiate the client on startup and reuse it across the entire
        application."""
        try:
            self.common_proxy = xmlrpc.client.ServerProxy(f"{self.url}/xmlrpc/2/common")
            self.uid = self.common_proxy.authenticate(
                self.db, self.username, self.password, {}
            )

            if not self.uid:
                raise Exception("Authentication failed with Odoo: No UID returned.")

            self.models_proxy = xmlrpc.client.ServerProxy(f"{self.url}/xmlrpc/2/object")

        except xmlrpc.client.Fault as e:
            raise Exception(
                f"Odoo XML-RPC Fault during connection: {e.faultString}"
            ) from e
        except ConnectionRefusedError as e:
            raise Exception(
                f"Could not connect to Odoo at {self.url}. Connection refused."
            ) from e
        except Exception as e:
            raise Exception(
                f"An error occurred while connecting to Odoo: {str(e)}"
            ) from e

    def execute(self, model, method, args_list, **kwargs):
        if not self.uid or not self.models_proxy:
            self._connect()

        try:
            result = self.models_proxy.execute_kw(
                self.db, self.uid, self.password, model, method, args_list, kwargs
            )
            return result

        except xmlrpc.client.Fault as e:
            print(f"Fault! {e}")
            raise Exception(f"Odoo XML-RPC Fault: {e.faultString}") from e
        except ConnectionRefusedError as e:
            raise Exception(
                f"Connection to Odoo at {self.url} lost or refused during operation."
            ) from e
        except Exception as e:
            print(e)
            raise Exception(
                f"An error occurred while communicating with Odoo: {str(e)}"
            ) from e

    def search_read(self, model, domain, fields, **kwargs):
        return self.execute(model, "search_read", args_list=[domain, fields], **kwargs)


CLIENT = OdooClient()


def get_invoice_addresses(user):
    """Used during organization creation: retrieves all invoice
    addresses the user owns or is connected to from the Odoo API."""
    # We’re building our conditions in order:
    #  - in exceptions, users may be using a billing account’s email
    #  - if the user is associated with an odoo user, return all billing
    #    addresses / organizations created by the user
    #  - if the user is associated with an odoo contact, return all billing
    #    addresses with the same parent_id
    email = user.email
    or_conditions = [("email", "ilike", email)]

    email = user if isinstance(user, str) else user.email
    odoo_users = CLIENT.search_read(
        model="res.users",
        domain=[("login", "=", email)],
        fields=["id"],
        limit=1,
    )
    if odoo_users and (uid := odoo_users[0].get("id")):
        or_conditions.append(("create_uid", "=", uid))

    odoo_contacts = CLIENT.search_read(
        model="res.partner",
        domain=[
            ("company_type", "=", "person"),
            ("type", "=", "contact"),
            ("email", "ilike", email),
        ],
        fields=["id", "parent_id"],
    )
    if odoo_contacts:
        for contact in odoo_contacts:
            or_conditions.append(("parent_id", "=", contact["parent_id"][0]))

    if len(or_conditions) > 1:
        or_conditions = ["|"] * (len(or_conditions) - 1) + or_conditions

    # The domain requires the partner to be an invoice address, that is:
    # Of the company_type=person, and type=invoice.
    # If we were searching for an existing organization, we would also have to
    # filter for parent_id=odoo_company_id
    domain = [
        ("company_type", "=", "person"),
        ("type", "=", "invoice"),
    ] + or_conditions

    try:
        invoice_addresses = CLIENT.search_read(
            model="res.partner",
            domain=domain,
            fields=ADDRESS_FIELDS,
        )
        return invoice_addresses or []
    except Exception:
        return []
