from django.shortcuts import redirect
from django.utils.translation import gettext_lazy as _
from django.views.generic import CreateView, DetailView
from rules.contrib.views import AutoPermissionRequiredMixin

from servala.core.models import BillingEntity, Organization
from servala.frontend.forms.organization import OrganizationCreateForm, OrganizationForm
from servala.frontend.views.mixins import HtmxUpdateView, OrganizationViewMixin


class OrganizationCreateView(AutoPermissionRequiredMixin, CreateView):
    form_class = OrganizationCreateForm
    model = Organization
    template_name = "frontend/organizations/create.html"

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["user"] = self.request.user
        return kwargs

    def form_valid(self, form):
        billing_choice = form.cleaned_data.get("billing_processing_choice")
        billing_entity = None

        if billing_choice == "new":
            billing_entity = BillingEntity.create_from_data(
                {
                    key[3:]: value
                    for key, value in form.cleaned_data.items()
                    if key.startswith("ba_")
                }
            )
        elif odoo_id := form.cleaned_data.get("existing_odoo_address_id"):
            billing_entity = BillingEntity.objects.filter(
                odoo_invoice_id=odoo_id
            ).first()

            if not billing_entity:
                billing_entity = BillingEntity.create_from_id(odoo_id)

        if not billing_entity:
            form.add_error(
                None,
                _(
                    "Could not determine or create the billing entity. Please check your input."
                ),
            )
            return self.form_invalid(form)

        form.instance.billing_entity = billing_entity
        instance = form.instance.create_organization(
            form.instance, owner=self.request.user
        )
        return redirect(instance.urls.base)


class OrganizationDashboardView(
    AutoPermissionRequiredMixin, OrganizationViewMixin, DetailView
):
    template_name = "frontend/organizations/dashboard.html"


class OrganizationUpdateView(OrganizationViewMixin, HtmxUpdateView):
    template_name = "frontend/organizations/update.html"
    form_class = OrganizationForm
    fragments = ("org-name", "org-name-edit")

    def get_success_url(self):
        return self.request.path
